package cn.fw.valhalla.controller.task;

import cn.fw.valhalla.component.producer.ClueChangeProducer;
import cn.fw.valhalla.component.producer.RenewalSwitchProducer;
import cn.fw.valhalla.domain.db.follow.FollowClue;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import cn.fw.valhalla.sdk.enums.CustomerFollowTypeEnum;
import cn.fw.valhalla.sdk.result.ClueChangeResult;
import cn.fw.valhalla.service.data.FollowClueService;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.data.redis.core.BoundSetOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * 线索变更后的通知事件
 *
 * @author : kurisu
 * @version : 1.0
 * @className : ClueChangeNoticeTask
 * @description : 线索变更后的通知事件
 * @date : 2022-12-07 11:45
 */
@Component
@Slf4j
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
public class ClueChangeNoticeTask {
    private final ClueChangeProducer clueChangeProducer;
    private final RenewalSwitchProducer renewalSwitchProducer;
    private final FollowClueService followClueService;
    private final StringRedisTemplate redisTemplate;
    @Value("${spring.cache.custom.global-prefix}:follow:clue:change")
    @Getter
    private String clueChangeKeyPrefix;

    @Autowired
    public ClueChangeNoticeTask(final ClueChangeProducer clueChangeProducer,
                                final RenewalSwitchProducer renewalSwitchProducer,
                                final FollowClueService followClueService,
                                final StringRedisTemplate redisTemplate) {
        this.clueChangeProducer = clueChangeProducer;
        this.renewalSwitchProducer = renewalSwitchProducer;
        this.followClueService = followClueService;
        this.redisTemplate = redisTemplate;
    }

    @Scheduled(initialDelay = 1000 * 30, fixedRate = 1000 * 60)
    public void sendStopNotice() {
        List<String> failList = new ArrayList<>();
        final BoundSetOperations<String, String> operations = redisTemplate.boundSetOps(generateStopKey());
        String pop;
        while ((pop = operations.pop()) != null) {
            try {
                final Long clueId = Long.valueOf(pop);
                FollowClue clue = followClueService.getById(clueId);
                if (Objects.nonNull(clue)) {
                    ClueChangeResult result = new ClueChangeResult();
                    result.setFrameNo(clue.getVin());
                    result.setGroupId(clue.getGroupId());
                    result.setChangeType(ClueChangeResult.ChangeType.STOP);
                    if (FollowTypeEnum.RM.equals(clue.getClueType())) {
                        result.setClueType(CustomerFollowTypeEnum.RM.getValue());
                    } else if (FollowTypeEnum.IR.equals(clue.getClueType())) {
                        result.setClueType(CustomerFollowTypeEnum.IR.getValue());
                        renewalSwitchProducer.send(clue.getVin(), clue.getEndTime().toLocalDate(), clue.getGroupId());
                    }
                    clueChangeProducer.send(result);
                }
            } catch (Exception ex) {
                failList.add(pop);
                log.error(ex.getMessage(), ex);
            }
        }
        if (!CollectionUtils.isEmpty(failList)) {
            operations.add(failList.toArray(new String[0]));
        }
    }

    private String generateStopKey() {
        return String.format("%s:%s", getClueChangeKeyPrefix(), "STOP");
    }
}
